@@ -471,6 +471,8 @@ def RESTGetInfrastructureProperty(infid=None, prop=None):
 
             data = InfrastructureManager.ExportInfrastructure(infid, delete, auth)
             return format_output(data, default_type="application/json", field_name="data")
+        elif prop == "authorization":
+            res = InfrastructureManager.GetInfrastructureOwners(infid, auth)
         else:
             return return_error(404, "Incorrect infrastructure property")
 
@@ -765,10 +767,16 @@ def RESTAddResource(infid=None):
                 return return_error(415, "Unsupported Media Type %s" % content_type)
 
         if remove_list:
-            InfrastructureManager.RemoveResource(infid, remove_list, auth, context)
+            removed_vms = InfrastructureManager.RemoveResource(infid, remove_list, auth, context)
+            if len(remove_list) != removed_vms:
+                logger.error("Error deleting resources %s (removed %s)" % (remove_list, removed_vms))
 
         vm_ids = InfrastructureManager.AddResource(infid, radl_data, auth, context)
 
+        # If there are no changes in the infra, launch a reconfigure
+        if not remove_list and not vm_ids and context:
+            InfrastructureManager.Reconfigure(infid, "", auth)
+
         # Replace the TOSCA document
         if tosca_data:
             sel_inf = InfrastructureManager.get_infrastructure(infid, auth)
@@ -778,7 +786,10 @@ def RESTAddResource(infid=None):
         for vm_id in vm_ids:
             res.append(get_full_url("/infrastructures/" + str(infid) + "/vms/" + str(vm_id)))
 
-        return format_output(res, "text/uri-list", "uri-list", "uri")
+        if not vm_ids and remove_list and len(remove_list) != removed_vms:
+            return return_error(404, "Error deleting resources %s (removed %s)" % (remove_list, removed_vms))
+        else:
+            return format_output(res, "text/uri-list", "uri-list", "uri")
     except DeletedInfrastructureException as ex:
         return return_error(404, "Error Adding resources: %s" % get_ex_error(ex))
     except IncorrectInfrastructureException as ex:
