@@ -29,6 +29,10 @@ except ImportError:
 from threading import Thread
 from stat import S_ISDIR
 
+from cryptography.hazmat.backends import default_backend
+from cryptography.hazmat.primitives.asymmetric import rsa
+from cryptography.hazmat.primitives import serialization
+
 
 class TimeOutException(Exception):
     """Timeout in the SSH execution"""
@@ -684,3 +688,22 @@ class SSH:
             res = status == 0
 
         return res
+
+    @staticmethod
+    def keygen():
+        """
+        Generates a keypair using the cryptography lib and returns a tuple (public, private)
+        """
+        key = rsa.generate_private_key(public_exponent=65537, key_size=2048,
+                                       backend=default_backend())
+
+        private = key.private_bytes(encoding=serialization.Encoding.PEM,
+                                    format=serialization.PrivateFormat.TraditionalOpenSSL,
+                                    encryption_algorithm=serialization.NoEncryption()
+                                    ).decode()
+
+        public = key.public_key().public_bytes(encoding=serialization.Encoding.OpenSSH,
+                                               format=serialization.PublicFormat.OpenSSH
+                                               ).decode()
+
+        return (public, private)
