@@ -23,7 +23,7 @@ from toscaparser.functions import Function, is_function, get_function, GetAttrib
 from toscaparser.elements.scalarunit import ScalarUnit_Size
 from IM.ansible_utils import merge_recipes
 from radl.radl import (system, deploy, network, Feature, Features, configure,
-                       contextualize_item, RADL, contextualize, ansible)
+                       contextualize_item, RADL, contextualize, ansible, description)
 
 
 class Tosca:
@@ -136,6 +136,11 @@ class Tosca:
                             Tosca.logger.warn("Unknown tyoe of token %s. Ignoring." % token_type)
                 radl.ansible_hosts = [ansible_host]
             elif root_type == "tosca.nodes.aisprint.FaaS.Function":
+                min_instances, _, default_instances, count, removal_list = self._get_scalable_properties(node)
+                num_instances = self._get_num_instances(count, default_instances, min_instances)
+                if num_instances not in [0, 1]:
+                    raise Exception("Scalability values of %s only allows 0 or 1." % root_type)
+
                 oscar_host = self._find_host_compute(node, self.tosca.nodetemplates,
                                                      "tosca.nodes.SoftwareComponent")
                 oscar_compute = self._find_host_compute(node, self.tosca.nodetemplates)
@@ -150,13 +155,22 @@ class Tosca:
                     service_password = self._final_function_result(oscar_host.get_property_value('password'),
                                                                    oscar_host)
 
-                    recipe = '  - tasks:\n'
-                    recipe += '    - include_tasks: utils/tasks/oscar_function.yml\n'
-                    recipe += '      vars:\n'
-                    recipe += '        oscar_endpoint: "%s"\n' % service_endpoint
-                    recipe += '        oscar_username: "oscar"\n'
-                    recipe += '        oscar_password: "%s"\n' % service_password
-                    recipe += "        oscar_service_json: '%s'\n" % service_json
+                    if num_instances <= 0:
+                        recipe = '  - tasks:\n'
+                        recipe += '    - include_tasks: utils/tasks/del_oscar_function.yml\n'
+                        recipe += '      vars:\n'
+                        recipe += '        oscar_endpoint: "%s"\n' % service_endpoint
+                        recipe += '        oscar_username: "oscar"\n'
+                        recipe += '        oscar_password: "%s"\n' % service_password
+                        recipe += "        oscar_service_name: '%s'\n" % service_json["name"]
+                    else:
+                        recipe = '  - tasks:\n'
+                        recipe += '    - include_tasks: utils/tasks/oscar_function.yml\n'
+                        recipe += '      vars:\n'
+                        recipe += '        oscar_endpoint: "%s"\n' % service_endpoint
+                        recipe += '        oscar_username: "oscar"\n'
+                        recipe += '        oscar_password: "%s"\n' % service_password
+                        recipe += "        oscar_service_json: '%s'\n" % json.dumps(service_json)
 
                     conf = configure("oscar_%s" % node.name, recipe)
                     radl.configures.append(conf)
@@ -165,14 +179,26 @@ class Tosca:
                 else:
                     # if not create a system
                     oscar_sys = self._gen_oscar_system(node)
-                    radl.systems.append(oscar_sys)
-                    conf = configure(node.name, None)
-                    radl.configures.append(conf)
-                    level = Tosca._get_dependency_level(node)
-                    cont_items.append(contextualize_item(node.name, conf.name, level))
-                    cloud_id = self._get_placement_property(oscar_sys.name, "cloud_id")
-                    dep = deploy(oscar_sys.name, 1, cloud_id)
-                    radl.deploys.append(dep)
+
+                    current_num_instances = self._get_current_num_instances(oscar_sys.name, inf_info)
+                    num_instances = num_instances - current_num_instances
+                    Tosca.logger.debug("User requested %d instances of type %s and there"
+                                       " are %s" % (num_instances, oscar_sys.name, current_num_instances))
+
+                    if num_instances < 0:
+                        vm_ids = self._get_vm_ids_to_remove(removal_list, num_instances, inf_info, oscar_sys)
+                        if vm_ids:
+                            all_removal_list.extend(vm_ids)
+                            Tosca.logger.debug("List of FaaS to delete: %s" % vm_ids)
+                    else:
+                        radl.systems.append(oscar_sys)
+                        conf = configure(node.name, None)
+                        radl.configures.append(conf)
+                        level = Tosca._get_dependency_level(node)
+                        cont_items.append(contextualize_item(node.name, conf.name, level))
+                        cloud_id = self._get_placement_property(oscar_sys.name, "cloud_id")
+                        dep = deploy(oscar_sys.name, 1, cloud_id)
+                        radl.deploys.append(dep)
             else:
                 if root_type == "tosca.nodes.Compute":
                     # Add the system RADL element
@@ -184,25 +210,17 @@ class Tosca:
                     # Add the deploy element for this system
                     min_instances, _, default_instances, count, removal_list = self._get_scalable_properties(
                         node)
-                    if count is not None:
-                        # we must check the correct number of instances to
-                        # deploy
-                        num_instances = count
-                    elif default_instances is not None:
-                        num_instances = default_instances
-                    elif min_instances is not None:
-                        num_instances = min_instances
-                    else:
-                        num_instances = 1
-
-                    current_num_instances = self._get_num_instances(sys.name, inf_info)
+                    num_instances = self._get_num_instances(count, default_instances, min_instances)
+                    current_num_instances = self._get_current_num_instances(sys.name, inf_info)
                     num_instances = num_instances - current_num_instances
                     Tosca.logger.debug("User requested %d instances of type %s and there"
                                        " are %s" % (num_instances, sys.name, current_num_instances))
 
-                    # TODO: Think about to check the IDs of the VMs
                     if num_instances < 0:
-                        all_removal_list.extend(removal_list[0:-num_instances])
+                        vm_ids = self._get_vm_ids_to_remove(removal_list, num_instances, inf_info, sys)
+                        if vm_ids:
+                            all_removal_list.extend(vm_ids)
+                            Tosca.logger.debug("List of VMs to delete: %s" % vm_ids)
 
                     if num_instances > 0:
                         cloud_id = self._get_placement_property(sys.name, "cloud_id")
@@ -236,8 +254,39 @@ class Tosca:
 
         self._check_private_networks(radl, inf_info)
 
+        infra_name = self.tosca.tpl.get('metadata', {}).get('infra_name')
+        if infra_name:
+            radl.description = description('desc')
+            radl.description.setValue('name', infra_name)
+
         return all_removal_list, self._complete_radl_networks(radl)
 
+    @staticmethod
+    def _get_num_instances(count, default_instances, min_instances):
+        # we must check the correct number of instances to deploy
+        if count is not None:
+            num_instances = count
+        elif default_instances is not None:
+            num_instances = default_instances
+        elif min_instances is not None:
+            num_instances = min_instances
+        else:
+            num_instances = 1
+        return num_instances
+
+    @staticmethod
+    def _get_vm_ids_to_remove(removal_list, num_instances, inf_info, sys):
+        if removal_list:
+            # TODO: Think about to check the IDs of the VMs
+            return [int(vm_id) for vm_id in removal_list[0:-num_instances]]
+        # if no removal list is set, delete the oldest VMs of this type
+        elif inf_info:
+            vm_list = inf_info.get_vm_list_by_system_name()
+            if sys.name in vm_list:
+                vms = sorted(vm_list[sys.name], key=lambda vm: vm.creation_date)
+                return [vm.im_id for vm in vms[0:-num_instances]]
+        return None
+
     @staticmethod
     def _check_private_networks(radl, inf_info):
         """
@@ -328,7 +377,7 @@ class Tosca:
         radl.deploys = fe + wn + priv
 
     @staticmethod
-    def _get_num_instances(sys_name, inf_info):
+    def _get_current_num_instances(sys_name, inf_info):
         """
         Get the current number of instances of system type name sys_name
         """
@@ -415,6 +464,7 @@ class Tosca:
         # This is the solution using endpoints
         net_provider_id = None
         dns_name = None
+        additional_dns_names = []
         additional_ip = None
         ports = {}
         endpoints = self._get_node_endpoints(node, nodetemplates)
@@ -446,6 +496,8 @@ class Tosca:
                     net_provider_id = network_name
             if cap_props and "dns_name" in cap_props:
                 dns_name = self._final_function_result(cap_props["dns_name"].value, node)
+            if cap_props and "additional_dns_names" in cap_props:
+                additional_dns_names = self._final_function_result(cap_props["additional_dns_names"].value, node)
             if cap_props and "private_ip" in cap_props:
                 private_ip = self._final_function_result(cap_props["private_ip"].value, node)
             if cap_props and "ports" in cap_props:
@@ -477,7 +529,7 @@ class Tosca:
             # If there are network nodes, use it to define system network
             # properties
             port_net = None
-            for net_name, ip, dns_name, num, additional_ip in nets:
+            for net_name, ip, dns_name, additional_dns_names, num, additional_ip in nets:
                 net = radl.get_network_by_id(net_name)
                 if not net:
                     raise Exception("Node %s with a port binded to a non existing network: %s." % (node.name,
@@ -491,6 +543,8 @@ class Tosca:
                     system.setValue('net_interface.%d.dns_name' % num, dns_name)
                 if additional_ip:
                     system.setValue('net_interface.%d.additional_ip' % num, additional_ip)
+                if additional_dns_names:
+                    system.setValue('net_interface.%d.additional_dns_names' % num, additional_dns_names)
 
                 if net.isPublic():
                     port_net = net
@@ -559,6 +613,9 @@ class Tosca:
                     public_net.setValue("outports", outports)
 
                 system.setValue('net_interface.%d.connection' % num_net, public_net.id)
+                # allways add the additional_dns_names in the public interface
+                if additional_dns_names:
+                    system.setValue('net_interface.%d.additional_dns_names' % num_net, additional_dns_names)
 
             if net_provider_id:
                 if private_net:
@@ -761,7 +818,7 @@ class Tosca:
                             script_content = resp.text
                             if resp.status_code != 200:
                                 raise Exception(resp.reason + "\n" + resp.text)
-                        except Exception as ex:
+                        except Exception:
                             raise Exception("Implementation file: '%s' is not located in the artifacts folder '%s' "
                                             "or in the artifacts remote url '%s'." % (implementation,
                                                                                       Tosca.ARTIFACTS_PATH,
@@ -1190,59 +1247,68 @@ class Tosca:
                         return vm.getPrivateIP()
             elif attribute_name == "endpoint":
                 if root_type == "tosca.nodes.aisprint.FaaS.Function":
-                    oscar_host = self._find_host_compute(node, self.tosca.nodetemplates,
-                                                         "tosca.nodes.SoftwareComponent")
-                    if host_node != node and oscar_host:
-                        # OSCAR function deployed in a deployed VM
-                        dns_host = self._final_function_result(oscar_host.get_property_value('dns_host'), oscar_host)
-                        if dns_host and dns_host.strip("'\""):
-                            return "https://%s" % dns_host
-
-                        if vm.getPublicIP():
-                            return "http://%s" % vm.getPublicIP()
-                        else:
-                            return "http://%s" % vm.getPrivateIP()
+                    if vm.getCloudConnector().type == "Lambda":
+                        # AWS Lambda function
+                        return vm.info.systems[0].get("function.api_url")
                     else:
-                        # OSCAR function deployed in a pre-deployed cluster or not dns_host set
-                        return vm.getCloudConnector().cloud.get_url()
+                        oscar_host = self._find_host_compute(node, self.tosca.nodetemplates,
+                                                             "tosca.nodes.SoftwareComponent")
+                        if host_node != node and oscar_host:
+                            # OSCAR function deployed in a deployed VM
+                            dns_host = self._final_function_result(oscar_host.get_property_value('dns_host'),
+                                                                   oscar_host)
+                            if dns_host and dns_host.strip("'\""):
+                                return "https://%s" % dns_host
+
+                            if vm.getPublicIP():
+                                return "http://%s" % vm.getPublicIP()
+                            else:
+                                return "http://%s" % vm.getPrivateIP()
+                        else:
+                            # OSCAR function deployed in a pre-deployed cluster or not dns_host set
+                            return vm.getCloudConnector().cloud.get_url()
 
                 Tosca.logger.warn("Attribute endpoint only supported in tosca.nodes.aisprint.FaaS.Function")
                 return None
             elif attribute_name == "credential":
                 if root_type == "tosca.nodes.aisprint.FaaS.Function":
-                    oscar_host = self._find_host_compute(node, self.tosca.nodetemplates,
-                                                         "tosca.nodes.SoftwareComponent")
-                    if host_node != node and oscar_host:
-                        # OSCAR function deployed in a deployed VM
-                        oscar_pass = self._final_function_result(oscar_host.get_property_value('password'), oscar_host)
-                        if oscar_pass and oscar_pass.strip("'\""):
-                            return self._get_object_values({"user": "oscar",
-                                                            "token_type": "password",
-                                                            "token": oscar_pass},
-                                                           attribute_params)
-
-                        Tosca.logger.warn("No password defined in tosca.nodes.indigo.OSCAR host node")
+                    if vm.getCloudConnector().type == "Lambda":
                         return None
                     else:
-                        # OSCAR function deployed in a pre-deployed cluster or not dns_host set
-                        if vm.getCloudConnector().auth:
-                            auth = vm.getCloudConnector().auth
-                            if 'username' in auth and 'password' in auth:
-                                return self._get_object_values({"user": auth["username"],
+                        oscar_host = self._find_host_compute(node, self.tosca.nodetemplates,
+                                                             "tosca.nodes.SoftwareComponent")
+                        if host_node != node and oscar_host:
+                            # OSCAR function deployed in a deployed VM
+                            oscar_pass = self._final_function_result(oscar_host.get_property_value('password'),
+                                                                     oscar_host)
+                            if oscar_pass and oscar_pass.strip("'\""):
+                                return self._get_object_values({"user": "oscar",
                                                                 "token_type": "password",
-                                                                "token": auth["password"]},
+                                                                "token": oscar_pass},
                                                                attribute_params)
-                            elif 'token' in auth:
-                                return self._get_object_values({"user": "",
-                                                                "token_type": "bearer",
-                                                                "token": auth["token"]},
-                                                               attribute_params)
-                            else:
-                                Tosca.logger.warn("No valid auth data in OSCAR connector")
-                                return None
 
-                        Tosca.logger.warn("No auth data in OSCAR connector")
-                        return None
+                            Tosca.logger.warn("No password defined in tosca.nodes.indigo.OSCAR host node")
+                            return None
+                        else:
+                            # OSCAR function deployed in a pre-deployed cluster or not dns_host set
+                            if vm.getCloudConnector().auth:
+                                auth = vm.getCloudConnector().auth
+                                if 'username' in auth and 'password' in auth:
+                                    return self._get_object_values({"user": auth["username"],
+                                                                    "token_type": "password",
+                                                                    "token": auth["password"]},
+                                                                   attribute_params)
+                                elif 'token' in auth:
+                                    return self._get_object_values({"user": "",
+                                                                    "token_type": "bearer",
+                                                                    "token": auth["token"]},
+                                                                   attribute_params)
+                                else:
+                                    Tosca.logger.warn("No valid auth data in OSCAR connector")
+                                    return None
+
+                            Tosca.logger.warn("No auth data in OSCAR connector")
+                            return None
 
                 Tosca.logger.warn("Attribute credential only supported in tosca.nodes.aisprint.FaaS.Function")
                 return None
@@ -1619,6 +1685,8 @@ class Tosca:
                     res.setValue("instance_tags", instance_tags)
                 else:
                     raise Exception("tags property must be a dictionary.")
+            elif prop.name == "instance_name" and prop.value:
+                res.setValue("instance_name", prop.value)
 
         property_map = {
             'architecture': 'cpu.arch',
@@ -1735,7 +1803,9 @@ class Tosca:
                     order = self._final_function_result(port.get_property_value('order'), port)
                     dns_name = self._final_function_result(port.get_property_value('dns_name'), port)
                     additional_ip = self._final_function_result(port.get_property_value('additional_ip'), port)
-                    nets.append((link, ip, dns_name, order, additional_ip))
+                    additional_dns_names = self._final_function_result(port.get_property_value('additional_dns_names'),
+                                                                       port)
+                    nets.append((link, ip, dns_name, additional_dns_names, order, additional_ip))
 
         return nets
 
@@ -1938,6 +2008,15 @@ class Tosca:
                     # this should never happen
                     Tosca.logger.warn("Property %s not expected. Ignoring." % prop.name)
 
+            if node.requirements:
+                deps = []
+                for r, n in node.relationships.items():
+                    if Tosca._is_derived_from(r, [r.DEPENDSON]):
+                        node_compute = self._find_host_compute(n, self.tosca.nodetemplates)
+                        deps.append(node_compute.name)
+                if deps:
+                    res.setValue('dependencies', deps)
+
         return res
 
     def _get_oscar_service_json(self, node):
@@ -1954,7 +2033,7 @@ class Tosca:
                 elif prop.name == 'memory':
                     if not value.endswith("B"):
                         value += "B"
-                    res[prop.name] = "%gMi" % ScalarUnit_Size(value).get_num_from_scalar_unit('MiB')
+                    res[prop.name] = "%dMi" % ScalarUnit_Size(value).get_num_from_scalar_unit('MiB')
                 elif prop.name == 'image':
                     if value.startswith('oscar://'):
                         url_image = urlparse(value)
@@ -1974,4 +2053,4 @@ class Tosca:
                     # this should never happen
                     Tosca.logger.warn("Property %s not expected. Ignoring." % prop.name)
 
-        return json.dumps(res)
+        return res
