@@ -22,9 +22,9 @@ import logging.handlers
 import logging.config
 import os
 import signal
-import subprocess
 import time
 import argparse
+import psutil
 
 from IM.request import Request, AsyncXMLRPCServer, get_system_queue
 from IM.config import Config
@@ -220,6 +220,12 @@ def ChangeInfrastructureAuth(inf_id, new_auth_data, overwrite, auth_data):
     return WaitRequest(request)
 
 
+def GetInfrastructureOwners(inf_id, auth_data):
+    request = IMBaseRequest.create_request(
+        IMBaseRequest.GET_INFRASTRUCTURE_OWNERS, (inf_id, auth_data))
+    return WaitRequest(request)
+
+
 def launch_daemon():
     """
     Launch the IM daemon
@@ -266,6 +272,7 @@ def launch_daemon():
     server.register_function(GetCloudImageList)
     server.register_function(GetCloudQuotas)
     server.register_function(ChangeInfrastructureAuth)
+    server.register_function(GetInfrastructureOwners)
 
     InfrastructureManager.logger.info(
         '************ Start Infrastructure Manager daemon (v.%s) ************' % version)
@@ -375,10 +382,10 @@ def im_stop():
 def get_childs(parent_id=None):
     if parent_id is None:
         parent_id = os.getpid()
-    ps_command = subprocess.Popen("ps -o pid --ppid %d --noheaders" % parent_id, shell=True, stdout=subprocess.PIPE)
-    ps_command.wait()
-    ps_output = str(ps_command.stdout.read())
-    childs = ps_output.strip().split("\n")[:-1]
+    childs = []
+    for proc in psutil.process_iter():
+        if not parent_id or parent_id == proc.ppid():
+            childs.append(proc.pid)
     if childs:
         res = childs
         for child in childs:
